@@ -34,7 +34,7 @@ pub use snarkvm_algorithms::fft::DensePolynomial as Polynomial;
 use snarkvm_fields::Field;
 use snarkvm_utilities::{error as error_fn, errors::SerializationError, serialize::*, FromBytes, ToBytes};
 
-use core::fmt::Debug;
+use core::{fmt::Debug, sync::atomic::AtomicBool};
 use rand_core::RngCore;
 
 #[cfg(not(feature = "std"))]
@@ -171,12 +171,10 @@ pub trait PolynomialCommitment<F: Field>: Sized + Clone + Debug {
         + From<Vec<Self::Proof>>
         + Into<Vec<Self::Proof>>
         + Debug;
-    /// The error type for the scheme.
-    type Error: snarkvm_utilities::error::Error + From<Error>;
 
     /// Constructs public parameters when given as input the maximum degree `degree`
     /// for the polynomial commitment scheme.
-    fn setup<R: RngCore>(max_degree: usize, rng: &mut R) -> Result<Self::UniversalParams, Self::Error>;
+    fn setup<R: RngCore>(max_degree: usize, rng: &mut R) -> Result<Self::UniversalParams, Error>;
 
     /// Specializes the public parameters for polynomials up to the given `supported_degree`
     /// and for enforcing degree bounds in the range `1..=supported_degree`.
@@ -185,7 +183,7 @@ pub trait PolynomialCommitment<F: Field>: Sized + Clone + Debug {
         supported_degree: usize,
         supported_hiding_bound: usize,
         enforced_degree_bounds: Option<&[usize]>,
-    ) -> Result<(Self::CommitterKey, Self::VerifierKey), Self::Error>;
+    ) -> Result<(Self::CommitterKey, Self::VerifierKey), Error>;
 
     /// Outputs a commitments to `polynomials`. If `polynomials[i].is_hiding()`,
     /// then the `i`-th commitment is hiding up to `polynomials.hiding_bound()` queries.
@@ -201,7 +199,18 @@ pub trait PolynomialCommitment<F: Field>: Sized + Clone + Debug {
         ck: &Self::CommitterKey,
         polynomials: impl IntoIterator<Item = &'a LabeledPolynomial<F>>,
         rng: Option<&mut dyn RngCore>,
-    ) -> Result<(Vec<LabeledCommitment<Self::Commitment>>, Vec<Self::Randomness>), Self::Error>;
+    ) -> Result<(Vec<LabeledCommitment<Self::Commitment>>, Vec<Self::Randomness>), Error> {
+        Self::commit_with_terminator(ck, polynomials, &AtomicBool::new(false), rng)
+    }
+
+    /// Like [`commit`] but with an added early termination signal, [`terminator`].
+    #[allow(clippy::type_complexity)]
+    fn commit_with_terminator<'a>(
+        ck: &Self::CommitterKey,
+        polynomials: impl IntoIterator<Item = &'a LabeledPolynomial<F>>,
+        terminator: &AtomicBool,
+        rng: Option<&mut dyn RngCore>,
+    ) -> Result<(Vec<LabeledCommitment<Self::Commitment>>, Vec<Self::Randomness>), Error>;
 
     /// On input a list of labeled polynomials and a query point, `open` outputs a proof of evaluation
     /// of the polynomials at the query point.
@@ -213,7 +222,7 @@ pub trait PolynomialCommitment<F: Field>: Sized + Clone + Debug {
         opening_challenge: F,
         rands: impl IntoIterator<Item = &'a Self::Randomness>,
         rng: Option<&mut dyn RngCore>,
-    ) -> Result<Self::Proof, Self::Error>
+    ) -> Result<Self::Proof, Error>
     where
         Self::Randomness: 'a,
         Self::Commitment: 'a;
@@ -228,7 +237,7 @@ pub trait PolynomialCommitment<F: Field>: Sized + Clone + Debug {
         opening_challenge: F,
         rands: impl IntoIterator<Item = &'a Self::Randomness>,
         rng: Option<&mut dyn RngCore>,
-    ) -> Result<Self::BatchProof, Self::Error>
+    ) -> Result<Self::BatchProof, Error>
     where
         Self::Randomness: 'a,
         Self::Commitment: 'a,
@@ -300,7 +309,7 @@ pub trait PolynomialCommitment<F: Field>: Sized + Clone + Debug {
         proof: &Self::Proof,
         opening_challenge: F,
         rng: &mut R,
-    ) -> Result<bool, Self::Error>
+    ) -> Result<bool, Error>
     where
         Self::Commitment: 'a;
 
@@ -314,7 +323,7 @@ pub trait PolynomialCommitment<F: Field>: Sized + Clone + Debug {
         proof: &Self::BatchProof,
         opening_challenge: F,
         rng: &mut R,
-    ) -> Result<bool, Self::Error>
+    ) -> Result<bool, Error>
     where
         Self::Commitment: 'a,
     {
@@ -369,7 +378,7 @@ pub trait PolynomialCommitment<F: Field>: Sized + Clone + Debug {
         opening_challenge: F,
         rands: impl IntoIterator<Item = &'a Self::Randomness>,
         rng: Option<&mut dyn RngCore>,
-    ) -> Result<BatchLCProof<F, Self>, Self::Error>
+    ) -> Result<BatchLCProof<F, Self>, Error>
     where
         Self::Randomness: 'a,
         Self::Commitment: 'a,
@@ -405,7 +414,7 @@ pub trait PolynomialCommitment<F: Field>: Sized + Clone + Debug {
         proof: &BatchLCProof<F, Self>,
         opening_challenge: F,
         rng: &mut R,
-    ) -> Result<bool, Self::Error>
+    ) -> Result<bool, Error>
     where
         Self::Commitment: 'a,
     {
@@ -474,7 +483,7 @@ pub trait PolynomialCommitment<F: Field>: Sized + Clone + Debug {
         query_set: &QuerySet<F>,
         opening_challenges: &dyn Fn(u64) -> F,
         rands: impl IntoIterator<Item = &'a Self::Randomness>,
-    ) -> Result<BatchLCProof<F, Self>, Self::Error>
+    ) -> Result<BatchLCProof<F, Self>, Error>
     where
         Self::Randomness: 'a,
         Self::Commitment: 'a;
@@ -489,7 +498,7 @@ pub trait PolynomialCommitment<F: Field>: Sized + Clone + Debug {
         proof: &BatchLCProof<F, Self>,
         opening_challenges: &dyn Fn(u64) -> F,
         rng: &mut R,
-    ) -> Result<bool, Self::Error>
+    ) -> Result<bool, Error>
     where
         Self::Commitment: 'a;
 }
@@ -557,7 +566,7 @@ pub mod tests {
         pub randomness: Vec<PC::Randomness>,
     }
 
-    pub fn bad_degree_bound_test<F, PC>() -> Result<(), PC::Error>
+    pub fn bad_degree_bound_test<F, PC>() -> Result<(), Error>
     where
         F: Field,
         PC: PolynomialCommitment<F>,
@@ -618,7 +627,7 @@ pub mod tests {
         Ok(())
     }
 
-    fn test_template<F, PC>(info: TestInfo) -> Result<Vec<TestComponents<F, PC>>, PC::Error>
+    fn test_template<F, PC>(info: TestInfo) -> Result<Vec<TestComponents<F, PC>>, Error>
     where
         F: Field,
         PC: PolynomialCommitment<F>,
@@ -739,7 +748,7 @@ pub mod tests {
         Ok(test_components)
     }
 
-    fn equation_test_template<F, PC>(info: TestInfo) -> Result<Vec<TestComponents<F, PC>>, PC::Error>
+    fn equation_test_template<F, PC>(info: TestInfo) -> Result<Vec<TestComponents<F, PC>>, Error>
     where
         F: Field,
         PC: PolynomialCommitment<F>,
@@ -908,7 +917,7 @@ pub mod tests {
         Ok(test_components)
     }
 
-    pub fn single_poly_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, PC::Error>
+    pub fn single_poly_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, Error>
     where
         F: Field,
         PC: PolynomialCommitment<F>,
@@ -925,7 +934,7 @@ pub mod tests {
         test_template::<F, PC>(info)
     }
 
-    pub fn linear_poly_degree_bound_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, PC::Error>
+    pub fn linear_poly_degree_bound_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, Error>
     where
         F: Field,
         PC: PolynomialCommitment<F>,
@@ -942,7 +951,7 @@ pub mod tests {
         test_template::<F, PC>(info)
     }
 
-    pub fn single_poly_degree_bound_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, PC::Error>
+    pub fn single_poly_degree_bound_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, Error>
     where
         F: Field,
         PC: PolynomialCommitment<F>,
@@ -959,7 +968,7 @@ pub mod tests {
         test_template::<F, PC>(info)
     }
 
-    pub fn quadratic_poly_degree_bound_multiple_queries_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, PC::Error>
+    pub fn quadratic_poly_degree_bound_multiple_queries_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, Error>
     where
         F: Field,
         PC: PolynomialCommitment<F>,
@@ -976,7 +985,7 @@ pub mod tests {
         test_template::<F, PC>(info)
     }
 
-    pub fn single_poly_degree_bound_multiple_queries_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, PC::Error>
+    pub fn single_poly_degree_bound_multiple_queries_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, Error>
     where
         F: Field,
         PC: PolynomialCommitment<F>,
@@ -993,7 +1002,7 @@ pub mod tests {
         test_template::<F, PC>(info)
     }
 
-    pub fn two_polys_degree_bound_single_query_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, PC::Error>
+    pub fn two_polys_degree_bound_single_query_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, Error>
     where
         F: Field,
         PC: PolynomialCommitment<F>,
@@ -1010,7 +1019,7 @@ pub mod tests {
         test_template::<F, PC>(info)
     }
 
-    pub fn full_end_to_end_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, PC::Error>
+    pub fn full_end_to_end_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, Error>
     where
         F: Field,
         PC: PolynomialCommitment<F>,
@@ -1027,7 +1036,7 @@ pub mod tests {
         test_template::<F, PC>(info)
     }
 
-    pub fn full_end_to_end_equation_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, PC::Error>
+    pub fn full_end_to_end_equation_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, Error>
     where
         F: Field,
         PC: PolynomialCommitment<F>,
@@ -1044,7 +1053,7 @@ pub mod tests {
         equation_test_template::<F, PC>(info)
     }
 
-    pub fn single_equation_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, PC::Error>
+    pub fn single_equation_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, Error>
     where
         F: Field,
         PC: PolynomialCommitment<F>,
@@ -1061,7 +1070,7 @@ pub mod tests {
         equation_test_template::<F, PC>(info)
     }
 
-    pub fn two_equation_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, PC::Error>
+    pub fn two_equation_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, Error>
     where
         F: Field,
         PC: PolynomialCommitment<F>,
@@ -1078,7 +1087,7 @@ pub mod tests {
         equation_test_template::<F, PC>(info)
     }
 
-    pub fn two_equation_degree_bound_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, PC::Error>
+    pub fn two_equation_degree_bound_test<F, PC>() -> Result<Vec<TestComponents<F, PC>>, Error>
     where
         F: Field,
         PC: PolynomialCommitment<F>,
